//
//  Bardecode.m
//  iBardecode
//
//  Copyright 2010 Softek Software Ltd. All rights reserved.
//

#import "Bardecode.h"


@implementation Bardecode

- (id) init
{
    if ( self = [super init] )
    {
		[self setLicenseKey:@""];
		[self setBarcodeValue:@""];
		[self setBarcodeType:@""];

    }
    return self;
}

- (NSString*) BarcodeValue {
    return BarcodeValue;
}

- (void) setBarcodeValue: (NSString *) input
{
	[BarcodeValue autorelease] ;
	BarcodeValue = [input retain] ;
}

- (NSString*) BarcodeType {
    return BarcodeType;
}

- (void) setBarcodeType: (NSString *) input
{
	[BarcodeType autorelease] ;
	BarcodeType = [input retain] ;
}

- (UIViewController*) Delegate {
	return Delegate;
}

- (void) setDelegate: (UIViewController *)input
{
	Delegate = input ;
}

- (void) setLicenseKey: (NSString *) input
{
	[LicenseKey autorelease] ;
	LicenseKey = [input retain] ;
}

- (void) setErrorMargin: (float) input
{
	ErrorMargin = input ;
}


-(int)ScanBarcodeFromBitmap:(CGImageRef)inImage
{
	void *hBarcode ;
	int n ;
	BITMAP bm ;
	char **bar_codes ;
	char **bar_codes_type ;
	void setVinParams(float newlimit1, float newlimit2, float newlimit3, float newerrorMargin) ;
	
	hBarcode = STCreateBarCodeSession() ;
	
	// Create the BITMAP structure from the CGImageRef
	bm.bmType = 0 ;
	bm.bmWidth = CGImageGetWidth(inImage) ; /* Pixel width */
	bm.bmHeight = CGImageGetHeight(inImage) ; /* Pixel height */
	bm.bmWidthBytes = CGImageGetBytesPerRow(inImage) ;
	bm.bmBitsPixel = CGImageGetBitsPerPixel(inImage) ;
	bm.bmPlanes = 1 ;
	//bm.bmBits = bits ;
	CFDataRef pixels = CGDataProviderCopyData(CGImageGetDataProvider(inImage));
	bm.bmBits = (void *) pixels ;
	
	
	// Set some parameters to control the barcode reading.
	
	uint16 val = 5 ;
	STSetParameter(hBarcode, ST_MIN_OCCURRENCE, &val) ;
	
	// A 30 day license can be obtained from sales@bardecode.com
	char *license = (char *) [LicenseKey cStringUsingEncoding:[NSString defaultCStringEncoding]] ;
	STSetParameter(hBarcode, ST_LICENSE, license) ;
	
	setVinParams(1.4, 1.0, 1.4, ErrorMargin) ;
	
	// Read the barcode from the image.
	
	n = STReadBarCodeFromBitmap(hBarcode, &bm, (float) 200, &bar_codes, &bar_codes_type, 0) ;
	
	// Create the string to pass back
	if (n)
	{
		[self setBarcodeValue:[NSString stringWithFormat:@"%s",bar_codes[0]]];
		[self setBarcodeType:[NSString stringWithFormat:@"%s",bar_codes_type[0]]];
	}
	
	STFreeBarCodeSession(hBarcode);
	
	CFRelease(pixels) ;
	
	return n ;
}

-(void) emptyResults
{
	[self setBarcodeValue:@""] ;
	[self setBarcodeType:@""] ;
}
	

-(void) ScanBarcodeFromCameraRoll
{
	[self emptyResults];
	scanTimer = nil ;
	
	picker = [[UIImagePickerController alloc] init];

	picker.delegate = self;
	
	picker.sourceType = UIImagePickerControllerSourceTypeSavedPhotosAlbum;

	[Delegate presentModalViewController:picker animated:YES];
	
}

-(void)onTimer                                                                                                    
{                                                                                                                                     
	scanTimer = nil ;

	extern CGImageRef UIGetScreenImage();
	CGImageRef image = UIGetScreenImage();
	
	int n = [self ScanBarcodeFromBitmap:image] ;

	
	if (++count > 20)
	{
		UIImage *img = [UIImage imageWithCGImage:image] ;
		UIImageWriteToSavedPhotosAlbum(img, self, @selector(image:didFinishSavingWithError:contextInfo:), nil);
		CGImageRelease(image);
		[picker dismissModalViewControllerAnimated:YES];
		[self sendNotification:@"BarcodeDidNotRead"];
		return ;
	}

	CGImageRelease(image);
	
	if (n) {
		// Process barcode
		AudioServicesPlaySystemSound(kSystemSoundID_Vibrate);
		[picker dismissModalViewControllerAnimated:YES];
		[self sendNotification:@"BarcodeDidRead"];
	}
	else {
		scanTimer = [NSTimer scheduledTimerWithTimeInterval:0.3 target:self selector:@selector(onTimer) userInfo:nil repeats:NO];
	}
}   

- (void)image:(UIImage *)image didFinishSavingWithError:(NSError *)error 
  contextInfo:(void *)contextInfo
{
    // Was there an error?
    if (error != NULL)
    {
		// Show error message...
		
    }
    else  // No errors
    {
		// Show message image successfully saved
    }
}


-(void) ScanBarcodeFromViewFinder
{
	[self emptyResults];
	picker = [[UIImagePickerController alloc] init];
	
	picker.delegate = self;
	
	picker.sourceType = UIImagePickerControllerSourceTypeCamera;
	picker.showsCameraControls = NO ;

	overlayView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 320, 480)];
	overlayView.opaque = NO;
	
	UIImageView *crosshairs = [[[UIImageView alloc] initWithImage:[UIImage imageNamed:@"overlay.png"]] autorelease];
	[overlayView addSubview:crosshairs];
	
	picker.cameraOverlayView = overlayView ;
	
	[Delegate presentModalViewController:picker animated:YES];
	
	count = 0 ;
	
	scanTimer = [NSTimer scheduledTimerWithTimeInterval:3.0 target:self selector:@selector(onTimer) userInfo:nil repeats:NO];
	
}

-(void) ScanBarcodeFromCamera
{
	[self emptyResults];
	scanTimer = nil;
	
	picker = [[UIImagePickerController alloc] init];
	
	picker.delegate = self;
	
	picker.sourceType = UIImagePickerControllerSourceTypeCamera;
	
	[Delegate presentModalViewController:picker animated:YES];
}

-(void) sendNotification: (NSString *)name
{
	if ([[self Delegate] respondsToSelector:@selector(BardecodeDidFinish:)]) {
		[[self Delegate] BardecodeDidFinish:[NSNotification notificationWithName:name object:self]];
		return ;
	}		
}

-(void)cancelTimer
{
	if ([scanTimer isValid]) {
		[scanTimer invalidate];
		[scanTimer release];
	}
}

- (void)imagePickerController:(UIImagePickerController *) pkr didFinishPickingMediaWithInfo:(NSDictionary *)info {
	[self cancelTimer] ;
	
	[pkr dismissModalViewControllerAnimated:YES];
	
	CGImageRef image = [(UIImage*) [info objectForKey:@"UIImagePickerControllerOriginalImage"] CGImage] ;

	int n = [self ScanBarcodeFromBitmap:image] ;
	
	if (n > 0)
		[self sendNotification:@"BarcodeDidRead"] ;
	else {
		[self sendNotification:@"BarcodeDidNotRead"] ;
	}	
	
}

- (void) imagePickerControllerDidCancel:(UIImagePickerController *) pkr
{
	[self cancelTimer];
	[pkr dismissModalViewControllerAnimated:YES];
	[self sendNotification:@"UserDidCancel"] ;
}

@end
